
# q-expansion of newform 600.2.y.b, downloaded from the LMFDB on 04 June 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 600
weight = 2
poly_data = [1, -1, 1, -1, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = [[151, [1, 0, 0, 0]], [301, [1, 0, 0, 0]], [401, [1, 0, 0, 0]], [577, [0, 0, 0, -1]]]
aps_data = [[0, 0, 0, 0], [0, 0, 0, 1], [0, -2, 1, -2], [0, 0, -2, 2], [-2, -2, 2, 2], [1, 1, 1, 0], [0, 1, 4, 1], [0, 4, 2, 4], [-4, 2, -2, 4], [5, -5, 0, 1], [0, -4, 2, -4], [5, -6, 5, 0], [-3, -1, -3, 0], [6, 0, 4, -4], [-6, 6, 0, 6], [9, -9, 0, -2], [-8, 4, -8, 0], [-10, 7, -7, 10], [0, 2, 8, 2], [0, 0, 0, -10], [4, 7, -7, -4], [0, 0, 0, 4], [0, 6, -10, 6], [-13, 16, -16, 13], [-1, 1, 0, 5], [11, 0, 5, -5], [6, -6, 0, 6], [-8, 0, 2, -2], [-3, 9, -3, 0], [-5, -4, -5, 0], [2, 10, -10, -2], [0, -6, 4, -6], [9, -5, 9, 0], [4, 10, -10, -4], [2, 0, 1, -1], [-8, 0, -6, 6], [17, 0, -1, 1], [-2, 2, -2, 0], [0, 4, -4, 4], [13, 2, -2, -13], [-18, 18, 0, 6], [0, -5, 14, -5], [-14, 4, -14, 0], [-8, 0, -3, 3], [-11, 11, 0, 18], [-6, 0, -12, 12], [16, -20, 20, -16], [14, -20, 20, -14], [4, -12, 12, -4], [-13, 13, 0, 18], [0, -5, 25, -5], [-12, 20, -20, 12], [9, -4, 9, 0], [12, 0, 10, -10], [12, 0, 11, -11], [10, -14, 10, 0], [0, -7, 5, -7], [6, -6, 0, -6], [19, -26, 26, -19], [0, 19, -15, 19], [0, -2, 12, -2], [7, 0, -11, 11], [-14, 0, -20, 20], [2, 4, -4, -2], [-12, 2, -12, 0], [0, 8, 14, 8], [0, -18, 2, -18], [-12, 6, -12, 0], [8, -8, 0, 10], [33, 0, 3, -3], [-4, 4, 0, 14], [2, 26, 2, 0], [0, -18, 6, -18], [-18, 18, -18, 18], [-4, 4, 0, -12], [0, -2, -22, -2], [10, -7, 7, -10], [16, -16, 0, -6], [24, 0, -5, 5], [-3, -8, -3, 0], [0, 0, 4, 0], [-17, 17, 0, -5], [0, 16, 10, 16], [0, 5, 15, 5], [10, -24, 24, -10], [-6, 0, 2, -2], [-11, 0, 1, -1], [22, 0, 8, -8], [9, 18, -18, -9], [-8, 18, -8, 0], [0, 10, 0, 10], [-6, 6, 0, -28], [2, -12, 2, 0], [0, 20, 0, 0], [8, 0, 6, -6], [0, 0, 0, 20], [7, -30, 7, 0], [-1, 1, 0, -28], [-10, 28, -28, 10], [-3, -26, -3, 0], [-26, 26, 0, 12], [-9, 0, 9, -9], [0, -4, 0, 0], [0, 1, -32, 1], [18, -18, 0, -14], [18, 10, -10, -18], [-2, 22, -2, 0], [-7, 0, 5, -5], [4, 0, -18, 18], [40, 0, 9, -9], [-34, 0, -12, 12], [1, -22, 1, 0], [0, 19, 9, 19], [0, -24, 4, -24], [0, 0, 10, 0], [0, -18, 0, 0], [-4, 0, 20, -20], [-10, 10, 0, 8], [-7, 7, 0, 9], [26, -22, 26, 0], [-22, -6, 6, 22], [-3, 16, -16, 3], [22, -6, 6, -22], [0, 0, 44, 0], [2, 40, 2, 0], [-30, 0, 1, -1], [-7, 29, -7, 0], [0, 2, 2, 2], [26, 4, -4, -26], [0, 4, -6, 4], [-26, 2, -2, 26], [-6, 0, 18, -18], [34, 0, 4, -4], [-14, 0, -29, 29], [28, -27, 27, -28], [0, 4, 38, 4], [-2, 29, -29, 2], [-28, 6, -28, 0], [1, -1, 0, -14], [11, -32, 11, 0], [-38, 48, -48, 38], [-28, 28, 0, 14], [-14, -16, 16, 14], [22, -18, 18, -22], [-21, 21, 0, -10], [10, 0, 0, -10], [-5, 5, 0, 10], [22, 0, 0, 0], [4, 32, 4, 0], [-22, 12, -22, 0], [-30, 15, -15, 30], [0, 28, 2, 28], [0, -16, 10, -16], [4, -18, 4, 0], [22, 0, 20, -20], [18, -6, 6, -18], [0, -18, 14, -18], [-23, 23, 0, 35], [29, -16, 29, 0], [-13, -22, -13, 0], [18, -18, 0, 12], [-7, 7, 0, 23], [0, 32, -16, 32], [6, -6, 0, -12], [-40, 25, -25, 40], [0, -16, -22, -16], [-2, 20, -2, 0], [4, -4, 0, -22]]
